@@ -6,16 +6,31 @@ import os
 import re
 import sys
 from argparse import ArgumentParser, ArgumentTypeError, SUPPRESS
+from enum import IntEnum
 from typing import List, Optional
 
 from . import (AbortDownloadException, BadCredentialsException, Instaloader, InstaloaderException,
-               InvalidArgumentException, Post, Profile, ProfileNotExistsException, StoryItem,
+               InvalidArgumentException, LoginException, Post, Profile, ProfileNotExistsException, StoryItem,
                TwoFactorAuthRequiredException, __version__, load_structure_from_file)
 from .instaloader import (get_default_session_filename, get_default_stamps_filename)
 from .instaloadercontext import default_user_agent
 from .lateststamps import LatestStamps
+try:
+    import browser_cookie3
+    bc3_library = True
+except ImportError:
+    bc3_library = False
 
 
+class ExitCode(IntEnum):
+    SUCCESS = 0
+    NON_FATAL_ERROR = 1
+    INIT_FAILURE = 2
+    LOGIN_FAILURE = 3
+    DOWNLOAD_ABORTED = 4
+    USER_ABORTED = 5
+    UNEXPECTED_ERROR = 99
+
 def usage_string():
     # NOTE: duplicated in README.rst and docs/index.rst
     argv0 = os.path.basename(sys.argv[0])
@@ -67,6 +82,56 @@ def filterstr_to_filterfunc(filter_str: str, item_type: type):
     return filterfunc
 
 
+def get_cookies_from_instagram(domain, browser, cookie_file='', cookie_name=''):
+    supported_browsers = {
+        "brave": browser_cookie3.brave,
+        "chrome": browser_cookie3.chrome,
+        "chromium": browser_cookie3.chromium,
+        "edge": browser_cookie3.edge,
+        "firefox": browser_cookie3.firefox,
+        "librewolf": browser_cookie3.librewolf,
+        "opera": browser_cookie3.opera,
+        "opera_gx": browser_cookie3.opera_gx,
+        "safari": browser_cookie3.safari,
+        "vivaldi": browser_cookie3.vivaldi,
+    }
+
+    if browser not in supported_browsers:
+        raise InvalidArgumentException("Loading cookies from the specified browser failed\n"
+                                       "Supported browsers are Brave, Chrome, Chromium, Edge, Firefox, LibreWolf, "
+                                       "Opera, Opera_GX, Safari and Vivaldi")
+
+    cookies = {}
+    browser_cookies = list(supported_browsers[browser](cookie_file=cookie_file))
+
+    for cookie in browser_cookies:
+        if domain in cookie.domain:
+            cookies[cookie.name] = cookie.value
+
+    if cookies:
+        print(f"Cookies loaded successfully from {browser}")
+    else:
+        raise LoginException(f"No cookies found for Instagram in {browser}, "
+                             f"Are you logged in succesfully in {browser}?")
+
+    if cookie_name:
+        return cookies.get(cookie_name, {})
+    else:
+        return cookies
+
+
+def import_session(browser, instaloader, cookiefile):
+    cookie = get_cookies_from_instagram('instagram', browser, cookiefile)
+    if cookie is not None:
+        instaloader.context.update_cookies(cookie)
+        username = instaloader.test_login()
+        if not username:
+            raise LoginException(f"Not logged in. Are you logged in successfully in {browser}?")
+        instaloader.context.username = username
+        print(f"{username} has been successfully logged in.")
+        print(f"Next time use --login={username} to reuse the same session.")
+
+
 def _main(instaloader: Instaloader, targetlist: List[str],
           username: Optional[str] = None, password: Optional[str] = None,
           sessionfile: Optional[str] = None,
@@ -78,7 +143,9 @@ def _main(instaloader: Instaloader, targetlist: List[str],
           fast_update: bool = False,
           latest_stamps_file: Optional[str] = None,
           max_count: Optional[int] = None, post_filter_str: Optional[str] = None,
-          storyitem_filter_str: Optional[str] = None) -> None:
+          storyitem_filter_str: Optional[str] = None,
+          browser: Optional[str] = None,
+          cookiefile: Optional[str] = None) -> ExitCode:
     """Download set of profiles, hashtags etc. and handle logging in and session files if desired."""
     # Parse and generate filter function
     post_filter = None
@@ -93,6 +160,11 @@ def _main(instaloader: Instaloader, targetlist: List[str],
     if latest_stamps_file is not None:
         latest_stamps = LatestStamps(latest_stamps_file)
         instaloader.context.log(f"Using latest stamps from {latest_stamps_file}.")
+    # load cookies if browser is not None
+    if browser and bc3_library:
+        import_session(browser.lower(), instaloader, cookiefile)
+    elif browser and not bc3_library:
+        raise InvalidArgumentException("browser_cookie3 library is needed to load cookies from browsers")
     # Login, if desired
     if username is not None:
         if not re.match(r"^[A-Za-z0-9._]+$", username):
@@ -108,6 +180,10 @@ def _main(instaloader: Instaloader, targetlist: List[str],
                 try:
                     instaloader.login(username, password)
                 except TwoFactorAuthRequiredException:
+                    # https://github.com/instaloader/instaloader/issues/1217
+                    instaloader.context.error("Warning: There have been reports of 2FA currently not working. "
+                                              "Consider importing session cookies from your browser with "
+                                              "--load-cookies.")
                     while True:
                         try:
                             code = input("Enter 2FA verification code: ")
@@ -117,14 +193,19 @@ def _main(instaloader: Instaloader, targetlist: List[str],
                             print(err, file=sys.stderr)
                             pass
             else:
-                instaloader.interactive_login(username)
+                try:
+                    instaloader.interactive_login(username)
+                except KeyboardInterrupt:
+                    print("\nInterrupted by user.", file=sys.stderr)
+                    return ExitCode.USER_ABORTED
         instaloader.context.log("Logged in as %s." % username)
     # since 4.2.9 login is required for geotags
     if instaloader.download_geotags and not instaloader.context.is_logged_in:
-        instaloader.context.error("Warning: Use --login to download geotags of posts.")
+        instaloader.context.error("Warning: Login is required to download geotags of posts.")
     # Try block for KeyboardInterrupt (save session on ^C)
     profiles = set()
     anonymous_retry_profiles = set()
+    exit_code = ExitCode.SUCCESS
     try:
         # Generate set of profiles, already downloading non-profile targets
         for target in targetlist:
@@ -215,7 +296,7 @@ def _main(instaloader: Instaloader, targetlist: List[str],
                                                                          ' '.join([p.username for p in profiles])))
         if instaloader.context.iphone_support and profiles and (download_profile_pic or download_posts) and \
            not instaloader.context.is_logged_in:
-            instaloader.context.log("Hint: Use --login to download higher-quality versions of pictures.")
+            instaloader.context.log("Hint: Login to download higher-quality versions of pictures.")
         instaloader.download_profiles(profiles,
                                       download_profile_pic, download_posts, download_tagged, download_igtv,
                                       download_highlights, download_stories,
@@ -230,8 +311,10 @@ def _main(instaloader: Instaloader, targetlist: List[str],
                                                    latest_stamps=latest_stamps)
     except KeyboardInterrupt:
         print("\nInterrupted by user.", file=sys.stderr)
+        exit_code = ExitCode.USER_ABORTED
     except AbortDownloadException as exc:
         print("\nDownload aborted: {}.".format(exc), file=sys.stderr)
+        exit_code = ExitCode.DOWNLOAD_ABORTED
     # Save session if it is useful
     if instaloader.context.is_logged_in:
         instaloader.save_session_to_file(sessionfile)
@@ -243,6 +326,8 @@ def _main(instaloader: Instaloader, targetlist: List[str],
         else:
             # Instaloader did not do anything
             instaloader.context.log("usage:" + usage_string())
+            exit_code = ExitCode.INIT_FAILURE
+    return exit_code
 
 
 def main():
@@ -258,17 +343,17 @@ def main():
                            help="Download profile. If an already-downloaded profile has been renamed, Instaloader "
                                 "automatically finds it by its unique ID and renames the folder likewise.")
     g_targets.add_argument('_at_profile', nargs='*', metavar="@profile",
-                           help="Download all followees of profile. Requires --login. "
+                           help="Download all followees of profile. Requires login. "
                                 "Consider using :feed rather than @yourself.")
     g_targets.add_argument('_hashtag', nargs='*', metavar='"#hashtag"', help="Download #hashtag.")
     g_targets.add_argument('_location', nargs='*', metavar='%location_id',
-                           help="Download %%location_id. Requires --login.")
+                           help="Download %%location_id. Requires login.")
     g_targets.add_argument('_feed', nargs='*', metavar=":feed",
-                           help="Download pictures from your feed. Requires --login.")
+                           help="Download pictures from your feed. Requires login.")
     g_targets.add_argument('_stories', nargs='*', metavar=":stories",
-                           help="Download the stories of your followees. Requires --login.")
+                           help="Download the stories of your followees. Requires login.")
     g_targets.add_argument('_saved', nargs='*', metavar=":saved",
-                           help="Download the posts that you marked as saved. Requires --login.")
+                           help="Download the posts that you marked as saved. Requires login.")
     g_targets.add_argument('_singlepost', nargs='*', metavar="-- -shortcode",
                            help="Download the post with the given shortcode")
     g_targets.add_argument('_json', nargs='*', metavar="filename.json[.xz]",
@@ -299,11 +384,11 @@ def main():
                         help='Download geotags when available. Geotags are stored as a '
                              'text file with the location\'s name and a Google Maps link. '
                              'This requires an additional request to the Instagram '
-                             'server for each picture. Requires --login.')
+                             'server for each picture. Requires login.')
     g_post.add_argument('-C', '--comments', action='store_true',
                         help='Download and update comments for each post. '
                              'This requires an additional request to the Instagram '
-                             'server for each post, which is why it is disabled by default.')
+                             'server for each post, which is why it is disabled by default. Requires login.')
     g_post.add_argument('--no-captions', action='store_true',
                         help='Do not create txt files.')
     g_post.add_argument('--post-metadata-txt', action='append',
@@ -317,11 +402,11 @@ def main():
     g_post.add_argument('--no-compress-json', action='store_true',
                         help='Do not xz compress JSON files, rather create pretty formatted JSONs.')
     g_prof.add_argument('-s', '--stories', action='store_true',
-                        help='Also download stories of each profile that is downloaded. Requires --login.')
+                        help='Also download stories of each profile that is downloaded. Requires login.')
     g_prof.add_argument('--stories-only', action='store_true',
                         help=SUPPRESS)
     g_prof.add_argument('--highlights', action='store_true',
-                        help='Also download highlights of each profile that is downloaded. Requires --login.')
+                        help='Also download highlights of each profile that is downloaded. Requires login.')
     g_prof.add_argument('--tagged', action='store_true',
                         help='Also download posts where each profile is tagged.')
     g_prof.add_argument('--igtv', action='store_true',
@@ -353,9 +438,14 @@ def main():
                                         'Instaloader can login to Instagram. This allows downloading private profiles. '
                                         'To login, pass the --login option. Your session cookie (not your password!) '
                                         'will be saved to a local file to be reused next time you want Instaloader '
-                                        'to login.')
+                                        'to login. Instead of --login, the --load-cookies option can be used to '
+                                        'import a session from a browser.')
     g_login.add_argument('-l', '--login', metavar='YOUR-USERNAME',
                          help='Login name (profile name) for your Instagram account.')
+    g_login.add_argument('-b', '--load-cookies', metavar='BROWSER-NAME',
+                         help='Browser name to load cookies from Instagram')
+    g_login.add_argument('-B', '--cookiefile', metavar='COOKIE-FILE',
+                         help='Cookie file of a profile to load cookies')
     g_login.add_argument('-f', '--sessionfile',
                          help='Path for loading and storing session key file. '
                               'Defaults to ' + get_default_session_filename("<login_name>"))
@@ -416,15 +506,15 @@ def main():
 
     args = parser.parse_args()
     try:
-        if args.login is None and (args.stories or args.stories_only):
-            print("--login=USERNAME required to download stories.", file=sys.stderr)
+        if (args.login is None and args.load_cookies is None) and (args.stories or args.stories_only):
+            print("Login is required to download stories.", file=sys.stderr)
             args.stories = False
             if args.stories_only:
-                raise SystemExit(1)
+                raise InvalidArgumentException()
 
         if ':feed-all' in args.profile or ':feed-liked' in args.profile:
-            raise SystemExit(":feed-all and :feed-liked were removed. Use :feed as target and "
-                             "eventually --post-filter=viewer_has_liked.")
+            raise InvalidArgumentException(":feed-all and :feed-liked were removed. Use :feed as target and "
+                                           "eventually --post-filter=viewer_has_liked.")
 
         post_metadata_txt_pattern = '\n'.join(args.post_metadata_txt) if args.post_metadata_txt else None
         storyitem_metadata_txt_pattern = '\n'.join(args.storyitem_metadata_txt) if args.storyitem_metadata_txt else None
@@ -434,15 +524,18 @@ def main():
                 post_metadata_txt_pattern = ''
                 storyitem_metadata_txt_pattern = ''
             else:
-                raise SystemExit("--no-captions and --post-metadata-txt or --storyitem-metadata-txt given; "
-                                 "That contradicts.")
+                raise InvalidArgumentException("--no-captions and --post-metadata-txt or --storyitem-metadata-txt "
+                                               "given; That contradicts.")
 
         if args.no_resume and args.resume_prefix:
-            raise SystemExit("--no-resume and --resume-prefix given; That contradicts.")
+            raise InvalidArgumentException("--no-resume and --resume-prefix given; That contradicts.")
         resume_prefix = (args.resume_prefix if args.resume_prefix else 'iterator') if not args.no_resume else None
 
         if args.no_pictures and args.fast_update:
-            raise SystemExit('--no-pictures and --fast-update cannot be used together.')
+            raise InvalidArgumentException('--no-pictures and --fast-update cannot be used together.')
+
+        if args.login and args.load_cookies:
+            raise InvalidArgumentException('--load-cookies and --login cannot be used together.')
 
         # Determine what to download
         download_profile_pic = not args.no_profile_pic or args.profile_pic_only
@@ -467,25 +560,37 @@ def main():
                              iphone_support=not args.no_iphone,
                              title_pattern=args.title_pattern,
                              sanitize_paths=args.sanitize_paths)
-        _main(loader,
-              args.profile,
-              username=args.login.lower() if args.login is not None else None,
-              password=args.password,
-              sessionfile=args.sessionfile,
-              download_profile_pic=download_profile_pic,
-              download_posts=download_posts,
-              download_stories=download_stories,
-              download_highlights=args.highlights,
-              download_tagged=args.tagged,
-              download_igtv=args.igtv,
-              fast_update=args.fast_update,
-              latest_stamps_file=args.latest_stamps,
-              max_count=int(args.count) if args.count is not None else None,
-              post_filter_str=args.post_filter,
-              storyitem_filter_str=args.storyitem_filter)
+        exit_code = _main(loader,
+                          args.profile,
+                          username=args.login.lower() if args.login is not None else None,
+                          password=args.password,
+                          sessionfile=args.sessionfile,
+                          download_profile_pic=download_profile_pic,
+                          download_posts=download_posts,
+                          download_stories=download_stories,
+                          download_highlights=args.highlights,
+                          download_tagged=args.tagged,
+                          download_igtv=args.igtv,
+                          fast_update=args.fast_update,
+                          latest_stamps_file=args.latest_stamps,
+                          max_count=int(args.count) if args.count is not None else None,
+                          post_filter_str=args.post_filter,
+                          storyitem_filter_str=args.storyitem_filter,
+                          browser=args.load_cookies,
+                          cookiefile=args.cookiefile)
         loader.close()
+        if loader.has_stored_errors:
+            exit_code = ExitCode.NON_FATAL_ERROR
+    except InvalidArgumentException as err:
+        print(err, file=sys.stderr)
+        exit_code = ExitCode.INIT_FAILURE
+    except LoginException as err:
+        print(err, file=sys.stderr)
+        exit_code = ExitCode.LOGIN_FAILURE
     except InstaloaderException as err:
-        raise SystemExit("Fatal error: %s" % err) from err
+        print("Fatal error: %s" % err)
+        exit_code = ExitCode.UNEXPECTED_ERROR
+    sys.exit(exit_code)
 
 
 if __name__ == "__main__":
